/**
 * PDBouncerPanel.java
 *
 * @author  Jeff Ondich
 * @version 2.0.0
 * date:    2013.11.03
 */

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class PDBouncerPanel extends PDPanel
                            implements PDObserver {

    private ArrayList<Sprite> spriteList;
    private Point spriteSize;
    private Random generator;
    private javax.swing.Timer timer;
    private PDNumberStrategy numberStrategy;

    /**
     * Default constructor. Sets up the PDPanel and the sprites.
     */
    public PDBouncerPanel() {
        this.generator = new Random();
     	this.numberStrategy = new PDArabicNumberStrategy();

       // Set up the frame window.
        this.setSize(500, 350);
        this.setBackground(new Color(30, 80, 30));
        this.setVisible(true);

        // Initialize the sprites.
        this.spriteList = new ArrayList<Sprite>();
        this.spriteSize = new Point(40, 40);
        this.spriteList.add(this.createSprite("Moose"));
        this.spriteList.add(this.createSprite("PDNumberSprite"));

        // Start the animation.
        this.timer = new javax.swing.Timer(40, new ActionListener() {
                        public void actionPerformed(ActionEvent evt) {
                            timer.stop();
                            step();
                            repaint();
                            timer.start();
                        }
                    });

        this.timer.start();
    }

    public void setNumberStrategy(PDNumberStrategy strategy) {
        this.numberStrategy = strategy;
    }

    /**
     * Private factory method that instantiates and initialize a sprite of the specified type
     * and attributes.
     *
     * @return an instance of the Sprite subclass named in spriteSubclassName
     * @param spriteSubclassName the name of the Sprite subclass you want instantiated
     */
    private Sprite createSprite(String spriteSubclassName) {
        Sprite sprite = null;
        try {
            sprite = (Sprite)Class.forName(spriteSubclassName).newInstance();
        } catch (Exception e) {
            System.err.println("Failed to create " + spriteSubclassName);
            System.err.println("    " + e.getMessage());
        }

        if (sprite != null) {
            sprite.setName(spriteSubclassName);
            sprite.setSize(this.spriteSize);
            Dimension windowSize = this.getSize();
            int x = this.generator.nextInt(windowSize.width);
            int y = this.generator.nextInt(windowSize.height);
            sprite.setPosition(new Point(x, y));
            int vx = this.generator.nextInt(9) - 4;
            int vy = this.generator.nextInt(9) - 4;
            sprite.setVelocity(new Point(vx, vy));
        }

        return sprite;
    }

    public void paint(Graphics g) {
        // Fill this window with its background color.  That is,
        // erase the whole thing. Calling "super.paint(g)" should
        // do the same thing, but some people are finding that they
        // end up with just a gray window if they do that.
        Dimension windowSize = this.getSize();
        g.setColor(this.getBackground());
        g.fillRect(0, 0, windowSize.width, windowSize.height);

        for (Sprite sprite : this.spriteList) {
            sprite.draw(g);
        }
    }

    public void step() {
        for (Sprite sprite : this.spriteList) {
            sprite.step();

            // Bounce the sprite off one or more walls, if appropriate.
            Dimension worldSize = this.getSize();
            Point velocity = sprite.getVelocity();
            Point position = sprite.getPosition();
            Point size = sprite.getSize();

            if (position.x < 0 && velocity.x < 0) {
                velocity.setLocation(-velocity.x, velocity.y);
            }

            if (position.x + size.x > worldSize.width && velocity.x > 0) {
                velocity.setLocation(-velocity.x, velocity.y);
            }

            if (position.y < 0 && velocity.y < 0) {
                velocity.setLocation(velocity.x, -velocity.y);
            }

            if (position.y + size.y > worldSize.height && velocity.y > 0) {
                velocity.setLocation(velocity.x, -velocity.y);
            }
        }
    }

    // PDPanel abstract methods.
    public void applyCachedValue() {
        for (Sprite sprite : this.spriteList) {
            sprite.setName(this.numberStrategy.getNumberString(this.cachedValue));
        }
    }

    // PDObserver interface.
    public void update(PDSubject subject) {
        this.cachedValue = subject.getValue();
        this.applyCachedValue();
    }
}

