/**
 * PDPanel.java
 *
 * An example Observer for the Pattern Demo.
 * 
 * @author  Jeff Ondich
 * @version 2.0.0
 * date:    2013.11.12
 */

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public abstract class PDPanel extends JPanel
                              implements PDObserver,
                                         ActionListener {

    private PDNumberStrategy numberStrategy;
    protected int cachedValue;

    public PDPanel() {
    	this.numberStrategy = new PDArabicNumberStrategy();
        this.cachedValue = 0;
    }
    
    public void setNumberStrategy(PDNumberStrategy strategy) {
    	this.numberStrategy = strategy;
        this.repaint();
    }

    public void actionPerformed(ActionEvent e) {
        JComboBox comboBox = (JComboBox)e.getSource();
        String strategyName = (String)comboBox.getSelectedItem();

        if ("Roman".equals(strategyName)) {
            this.setNumberStrategy(new PDRomanNumberStrategy());

        } else if ("Arabic".equals(strategyName)) {
            this.setNumberStrategy(new PDArabicNumberStrategy());

        } else if ("English".equals(strategyName)) {
            // Nothing yet.
        }

        this.applyCachedValue();
    }

    public abstract void applyCachedValue();
}

